==> org-lsst-ccs-subsystem-ocsbridge-main-2.7.2-sources.jar: org/lsst/ccs/subsystem/ocsbridge/OCSCommandExecutor.java <==
package org.lsst.ccs.subsystem.ocsbridge;

import org.lsst.ccs.subsystem.ocsbridge.util.State;
import java.time.Duration;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.sal.camera.states.CCSCommandStateEvent.CCSCommandState;
import org.lsst.sal.camera.CameraCommand;
import org.lsst.sal.camera.CameraEvent;
import org.lsst.sal.camera.CameraTelemetry;
import org.lsst.sal.camera.event.HeartbeatEvent;

/**
 * A simple abstraction layer to make it possible to run ocsbridge without an actual 
 * SAL implementation for testing. OCSCommandExecutor is not a good name for this, as it also 
 * deals with sending events and telemetry. 
 * @author tonyj
 */
public class OCSCommandExecutor implements OCSTelemetrySender, OCSEventSender {

    private static final Logger LOG = Logger.getLogger(OCSCommandExecutor.class.getName());
    private static final String UNKNOWN_VERSION = "Unknown";
    private final OCSBridge bridge;

    OCSCommandExecutor(OCSBridge bridge) {
        this.bridge = bridge;
    }

    void executeCommand(OCSExecutor command) {
        LOG.log(Level.INFO, "Received command: {0}", command);
        State<CCSCommandState> commandState = bridge.getCommandState();
        if (!commandState.isInState(CCSCommandState.IDLE)) {
            rejectCommand(command, "Command state not idle");
        } else {
            try {
                Duration timeout = command.testPreconditions();
                commandState.setState(CCSCommandState.BUSY);
                if (!timeout.isZero()) {
                    acknowledgeCommand(command, timeout);
                }
                command.execute();
                reportComplete(command);
            } catch (PreconditionsNotMet ex) {
                rejectCommand(command, ex.getMessage());
            } catch (Exception ex) {
                reportError(command, ex);
            } finally {
                commandState.setState(CCSCommandState.IDLE);
            }
        }
    }

    protected void rejectCommand(OCSExecutor command, String reason) {
        long rejectTime = System.nanoTime();
        LOG.log(Level.INFO, "Reject command: {0} because {1} in {2}ns", new Object[]{command, reason, rejectTime-command.startTime});
    }

    protected void acknowledgeCommand(OCSExecutor command, Duration timeout) {
        long ackTime = System.nanoTime();
        LOG.log(Level.INFO, "Acknowledge command: {0} timeout {1} in {2}ns", new Object[]{command, timeout, ackTime-command.startTime});
    }

    protected void reportError(OCSExecutor command, Exception ex) {
        LOG.log(Level.WARNING, "Command failed: " + command, ex);
    }

    protected void reportComplete(OCSExecutor command) {
        long completeTime = System.nanoTime();
        LOG.log(Level.INFO, "Command complete: {0} in {1}ns", new Object[]{command, completeTime-command.startTime});
    }

    /**
     * Send event, designed to be overridden in subclasses.
     */
    @Override
    public void sendEvent(CameraEvent event) {
        Level level = event instanceof HeartbeatEvent ? Level.FINE : Level.INFO;
        LOG.log(level, "OCS Event -> {0}", event);
    }
    
    @Override
    public void sendTelemetry(CameraTelemetry telemetry) {
        LOG.log(Level.FINE, "OCS Telemetry -> {0}", telemetry);
    }
    
    String getSALVersion() {
        return UNKNOWN_VERSION;
    }

    String getXMLVersion() {
        return UNKNOWN_VERSION;
    }
    
    String getOSPLVersion() {
        return UNKNOWN_VERSION;
    }

    /**
     * A base class for all OCS command executors
     *
     * @author tonyj
     */
    public static abstract class OCSExecutor {

        private final CameraCommand cmd;
        private final long startTime;
        
        OCSExecutor(CameraCommand command) {
            this.cmd = command;
            this.startTime = System.nanoTime();
        }

        /**
         * Check preconditions, and estimate the command duration.
         *
         * @throws PreconditionsNotMet If the preconditions are not met
         * @return The estimated duration of the command (can be ZERO)
         */
        abstract Duration testPreconditions() throws PreconditionsNotMet;

        /**
         * Actually perform the command
         */
        abstract void execute() throws Exception;

//        public int getCmdId() {
//            return cmd.getCmdId();
//        }

        public CameraCommand getCommand() {
            return cmd;
        }

        public long getStartTime() {
            return startTime;
        }

        @Override
        public String toString() {
            return cmd.toString();
        }

    }

    static class PreconditionsNotMet extends Exception {

        private static final long serialVersionUID = 2388630285363617708L;

        public PreconditionsNotMet(String reason) {
            super(reason);
        }
    }
}

==> org-lsst-ccs-subsystem-ocsbridge-main-2.8.3-sources.jar: org/lsst/ccs/subsystem/ocsbridge/OCSCommandExecutor.java <==
package org.lsst.ccs.subsystem.ocsbridge;

import org.lsst.ccs.subsystem.ocsbridge.util.State;
import java.time.Duration;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.sal.camera.states.CCSCommandStateEvent.CCSCommandState;
import org.lsst.sal.camera.CameraCommand;
import org.lsst.sal.camera.CameraEvent;
import org.lsst.sal.camera.CameraTelemetry;
import org.lsst.sal.camera.event.HeartbeatEvent;

/**
 * A simple abstraction layer to make it possible to run ocsbridge without an actual 
 * SAL implementation for testing. OCSCommandExecutor is not a good name for this, as it also 
 * deals with sending events and telemetry. 
 * @author tonyj
 */
public class OCSCommandExecutor implements OCSTelemetrySender, OCSEventSender {

    private static final Logger LOG = Logger.getLogger(OCSCommandExecutor.class.getName());
    private static final String UNKNOWN_VERSION = "Unknown";
    private final OCSBridge bridge;

    OCSCommandExecutor(OCSBridge bridge) {
        this.bridge = bridge;
    }

    void executeCommand(OCSExecutor command) {
        LOG.log(Level.INFO, "Received command: {0}", command);
        State<CCSCommandState> commandState = bridge.getCommandState();
        if (!commandState.isInState(CCSCommandState.IDLE)) {
            rejectCommand(command, "Command state not idle");
        } else {
            try {
                Duration timeout = command.testPreconditions();
                commandState.setState(CCSCommandState.BUSY);
                if (timeout != null && !timeout.isZero()) {
                    acknowledgeCommand(command, timeout);
                }
                command.execute();
                reportComplete(command);
            } catch (PreconditionsNotMet ex) {
                rejectCommand(command, ex.getMessage());
            } catch (Exception ex) {
                reportError(command, ex);
            } finally {
                commandState.setState(CCSCommandState.IDLE);
            }
        }
    }

    protected void rejectCommand(OCSExecutor command, String reason) {
        long rejectTime = System.nanoTime();
        LOG.log(Level.INFO, "Reject command: {0} because {1} in {2}ns", new Object[]{command, reason, rejectTime-command.startTime});
    }

    protected void acknowledgeCommand(OCSExecutor command, Duration timeout) {
        long ackTime = System.nanoTime();
        LOG.log(Level.INFO, "Acknowledge command: {0} timeout {1} in {2}ns", new Object[]{command, timeout, ackTime-command.startTime});
    }

    protected void reportError(OCSExecutor command, Exception ex) {
        LOG.log(Level.WARNING, "Command failed: " + command, ex);
    }

    protected void reportComplete(OCSExecutor command) {
        long completeTime = System.nanoTime();
        LOG.log(Level.INFO, "Command complete: {0} in {1}ns", new Object[]{command, completeTime-command.startTime});
    }

    /**
     * Send event, designed to be overridden in subclasses.
     */
    @Override
    public void sendEvent(CameraEvent event) {
        Level level = event instanceof HeartbeatEvent ? Level.FINE : Level.INFO;
        LOG.log(level, "OCS Event -> {0}", event);
    }
    
    @Override
    public void sendTelemetry(CameraTelemetry telemetry) {
        LOG.log(Level.FINE, "OCS Telemetry -> {0}", telemetry);
    }
    
    String getSALVersion() {
        return UNKNOWN_VERSION;
    }

    String getXMLVersion() {
        return UNKNOWN_VERSION;
    }
    
    String getOSPLVersion() {
        return UNKNOWN_VERSION;
    }

    /**
     * A base class for all OCS command executors
     *
     * @author tonyj
     */
    public static abstract class OCSExecutor {

        private final CameraCommand cmd;
        private final long startTime;
        
        OCSExecutor(CameraCommand command) {
            this.cmd = command;
            this.startTime = System.nanoTime();
        }

        /**
         * Check preconditions, and estimate the command duration.
         *
         * @throws PreconditionsNotMet If the preconditions are not met
         * @return The estimated duration of the command (can be ZERO)
         */
        abstract Duration testPreconditions() throws PreconditionsNotMet;

        /**
         * Actually perform the command
         */
        abstract void execute() throws Exception;

//        public int getCmdId() {
//            return cmd.getCmdId();
//        }

        public CameraCommand getCommand() {
            return cmd;
        }

        public long getStartTime() {
            return startTime;
        }

        @Override
        public String toString() {
            return cmd.toString();
        }

    }

    static class PreconditionsNotMet extends Exception {

        private static final long serialVersionUID = 2388630285363617708L;

        public PreconditionsNotMet(String reason) {
            super(reason);
        }
    }
}
